package ch.unibe.piccola;

import java.io.PrintStream;
import java.net.URL;
import ch.unibe.piccola.vm.Form;

/**
 * Runtime representation of the Piccola Environment.
 *
 * @author   Stefan Kneubuehl
 * @version  3.6c (May 11, 2003)
**/
public class RuntimeHost
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * The constant representing the boolean value <code>true</code>.
    **/
    public final boolean TRUE = true;

    /**
     * The constant representing the boolean value <code>false</code>.
    **/
    public final boolean FALSE = false;

    public final void exit()
    {
        m_host.abort();
    }

    /**
     * Returns the URL of the specified resource.
    **/
    public final URL findResource(String name)
    {
        return m_host.findResource(name);
    }

    /**
     * Looks up the Piccola system property with the speicified key.
     *
     * @see ch.unibe.piccola.Host#getProperty(String)
    **/
    public final String getProperty(String key)
    {
        return m_host.getProperty(key);
    }

    /**
     * Creates and returns a new channel.
    **/
    public final Form newChannel()
    {
        return m_host.newChannel();
    }

    public final Form newClass(Class cl)
    {
        return m_host.newClass(cl);
    }

    public final Form newClass(String className) throws ClassNotFoundException
    {
        return m_host.newClass(className);
    }

    public final void setExceptionHandler(Form value)
    {
        m_host.setExceptionHandler(value);
    }

    public final void setCallDynamic(Form value)
    {
        m_host.setCallDynamic(value);
    }

    public final Form newVariable(Form value)
    {
        return m_host.newVariable(value);
    }

    public final void print(String s)
    {
        m_host.print(s);
    }

    public final void println(String s)
    {
        m_host.println(s);
    }

    public final void registerWrapper(String className, Form wrapper)
    {
        m_host.registerPiccolaWrapper(className, wrapper);
    }

    public final Form serviceFromString(String code, Form staticContext)
    {
        return m_host.serviceFromString(code, staticContext);
    }

    public final Form serviceFromURL(URL url, Form staticContext)
    {
        return m_host.serviceFromURL(url, staticContext);
    }

    public final void setAutoAbort(boolean value)
    {
        m_host.setAutoAbort(value);
    }

    public final String toString()
    {
        return "JPiccola Runtime Environment " + m_host.getProperty("piccola.version");
    }

    public final String typeFromString(String code)
    {
        return m_host.typeFromString(code);
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    RuntimeHost(Host host)
    {
        m_host = host;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Host m_host;

}

