package ch.unibe.piccola.bytecode;

import org.apache.bcel.Constants;
import org.apache.bcel.generic.*;
import java.util.AbstractList;
import java.util.ArrayList;

/**
 * @author  $Id: ArrayFactory.java,v 3.5 2003/01/10 13:46:24 kneubuhl Exp $
**/

public class ArrayFactory implements Constants{

	public ClassGen createArrayOfClassName(String className) {
		Class aClass = TypeConverter.getClass(className);
		return createArrayOfClass(aClass);
	}

	protected ClassGen createArrayOfClass(Class aClass) {
		Type elementType = TypeConverter.class2bcelType(aClass);
	
		String className = "Gen" + getClassName(elementType.toString()) + "Array";
		className = "ch.unibe.piccola.bridge." + className;
		
		ClassGen cg = new ClassGen(className, "java.lang.Object", "<generated>",
                                ACC_PUBLIC | ACC_SUPER | ACC_STATIC, null);
		
	    ConstantPoolGen cp = cg.getConstantPool();
        InstructionList il = new InstructionList();  
		
		MethodGen mg = new MethodGen( ACC_PUBLIC,
                                new ArrayType(elementType, 1),
                                new Type[] {
									new ObjectType("java.util.ArrayList"),
									Type.INT
								}, null, 
								"convertToArray", className,
                                il, cp);
		
		InstructionFactory factory = new InstructionFactory(cg);
		il.append(factory.createLoad(Type.INT, 2));
		il.append((Instruction) factory.createNewArray(elementType, (short) 1));
		il.append(factory.createStore(Type.OBJECT, 3));
		il.append(new PUSH(cp, 0));
		il.append(factory.createStore(Type.INT,4));
		
		GOTO jmp = new GOTO(null);
		il.append(jmp);

		InstructionHandle label2 = il.append(factory.createLoad(Type.OBJECT, 3));
		il.append(factory.createLoad(Type.INT, 4));
		il.append(factory.createLoad(Type.OBJECT, 1));
		il.append(factory.createLoad(Type.INT, 4));
		il.append(factory.createInvoke("java.util.ArrayList", "get",
								Type.OBJECT, new Type[] {Type.INT},
								INVOKEVIRTUAL));

		if (aClass.isPrimitive()) {
			String classForPrimitiveType = TypeConverter.getClassForPrimitiveType(aClass);
			
			ObjectType classToCastTo = new ObjectType(classForPrimitiveType);
			il.append(factory.createCheckCast(classToCastTo));
			
			String methodName =	aClass.getName()+"Value";
			// Invoke <primType>Value() method
			il.append(factory.createInvoke(classForPrimitiveType, methodName,
									elementType, Type.NO_ARGS,	
									Constants.INVOKEVIRTUAL));		
		} else {
			il.append(factory.createCheckCast((ReferenceType) elementType));
		}
		
		il.append(factory.createArrayStore(elementType));
		il.append(new IINC(4, 1));
	
 		InstructionHandle label = il.append(factory.createLoad(Type.INT, 4));
		jmp.setTarget(label);
		il.append(factory.createLoad(Type.INT, 2));
						
		il.append(factory.createBranchInstruction(Constants.IF_ICMPLT, label2));
		
		il.append(factory.createLoad(Type.OBJECT, 3));
		il.append(factory.createReturn(Type.OBJECT));
		
		mg.setMaxStack();
		mg.setMaxLocals();
		
        cg.addMethod(mg.getMethod());
        il.dispose();
        
		cg.addEmptyConstructor(ACC_PUBLIC);

		return cg;
	}

	// Allows looping over arrays of object
	public void loop(Object[] array) {
		for(int i=0; i<array.length; i++) {
			System.out.println(array[i]);
		}
	}

	private String getClassName(String fullClassName) {
         return fullClassName.substring(fullClassName.lastIndexOf('.')+1,
						 				fullClassName.length());
    }
	
	/* * For decompiling purposes only * *
	public int[] intArray(AbstractList list, int size) {
		int[] array = new int[size];
		for(int i = 0; i<size; i++) {
			array[i] = ((Integer) list.get(i)).intValue();
		}
		return array;
	}
	
	public double[] doubleArray(AbstractList list, int size) {
		double[] array = new double[size];
		for(int i = 0; i<size; i++) {
			array[i] = ((Double) list.get(i)).doubleValue();
		}
		return array;
	}

	public String[] stringArray(AbstractList list, int size) {
		String[] array = new String[size];
		for(int i = 0; i<size; i++) {
			array[i] = ((String) list.get(i));
		}
		Object[] objArray = array;
		return array;
	}	
	*/
}


