package ch.unibe.piccola.bytecode;

import ch.unibe.piccola.code.CodeFactory;
import ch.unibe.piccola.vm.Form;
import ch.unibe.piccola.vm.FormFactory;
import ch.unibe.piccola.vm.PiccolaService;
import ch.unibe.piccola.vm.VMPool;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import java.lang.reflect.*;
import java.util.ArrayList;
import org.apache.bcel.Constants;

public class ByteCodeFactoryClassTest extends TestCase implements Constants, FactoryConstants {

	// test suite

    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(suite());
    }


    public static TestSuite suite()
    {
        return new TestSuite(ByteCodeFactoryClassTest.class);
    }


    public ByteCodeFactoryClassTest(String name)
    {
        super(name);
    }

	 // testing environment

    public void setUp()
    {
		bcf = new ByteCodeFactory();
		host = new VMPool();
		ff = new FormFactory();
		sf = new CodeFactory();
	}

    private Form newService(CodeFactory sf)
    {
        return ff.serviceForm(new PiccolaService(host, sf.code(), ff.emptyForm()));
    }

	private ByteCodeFactory bcf;

	private FormFactory ff;

	private VMPool host;

	private CodeFactory sf;

	// tests

	public void testSubclassing() {
		bcf.generateSubClass("ch.unibe.piccola.bytecode.ClassTest");
		Class aClass = bcf.loadClass();
		assertNotNull(aClass);

		Form ext = this.createSubclassForm();
		Object obj = this.instantiate(aClass, ext);

		//Now let's test
		assertEquals("GenClass1", ((ClassTest) obj).getClassName());
		assertEquals(42, ((ClassTest) obj).getVersion());
		assertEquals(false, ((ClassTest) obj).isVersionOne());

	}

	public void testSubclassingByHand() {
		String className = "ch.unibe.piccola.bytecode.GenByHandClass1";
		String superClassName = "ch.unibe.piccola.bytecode.ClassTest";
		int classModifier = ACC_PUBLIC;
		ArrayList methodDeclarationList = new ArrayList();

		String methodName = "getClassName";
		int methodModifier = ACC_PUBLIC;
		Class returnType = TypeConverter.getClass("java.lang.String");
		ArrayList argumentTypeList = new ArrayList();
		MethodDeclaration methodDeclaration = new MethodDeclaration(methodName, methodModifier,
									     					 returnType, argumentTypeList);
		methodDeclarationList.add(methodDeclaration);

		methodName = "getVersion";
		methodModifier = ACC_PUBLIC;

		returnType = TypeConverter.getClass("int");
		argumentTypeList = new ArrayList();
		methodDeclaration = new MethodDeclaration(methodName, methodModifier, returnType, argumentTypeList);
		methodDeclarationList.add(methodDeclaration);

		methodName = "isVersionOne";
		methodModifier = ACC_PUBLIC;
		returnType = TypeConverter.getClass("boolean");
		argumentTypeList = new ArrayList();
		methodDeclaration = new MethodDeclaration(methodName, methodModifier, returnType, argumentTypeList);
		methodDeclarationList.add(methodDeclaration);

		bcf.generateClass(className, superClassName, null, classModifier, methodDeclarationList);
		Class aClass = bcf.loadClass();
		assertNotNull(aClass);

		Form ext = this.createSubclassForm();
		Object obj = this.instantiate(aClass, ext);

		//Now let's test
		assertEquals("GenClass1", ((ClassTest) obj).getClassName());
		assertEquals(42, ((ClassTest) obj).getVersion());
		assertEquals(false, ((ClassTest) obj).isVersionOne());
	}

	private Form createSubclassForm() {
		sf.clear();
		sf.emitJavaObject(null, "GenClass1");
		sf.emitExit(null);
		Form a = newService(sf);
        Form aBindA = a.bind("getClassName");

		sf.clear();
		sf.emitJavaObject(null, new Integer(42));
        sf.emitExit(null);
		Form b = newService(sf);
        Form bBindB = b.bind("getVersion");

		sf.clear();
		sf.emitJavaObject(null, new Boolean(false));
        sf.emitExit(null);
		Form c = newService(sf);
        Form cBindC = c.bind("isVersionOne");

        sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form x = newService(sf);
        Form xBindX = x.bind("initialize");

		Form ext = aBindA.extend(bBindB).extend(cBindC).extend(xBindX);
		return ext;
	}

	public void testStringStack() {
		String className = "ch.unibe.piccola.bytecode.StringStack";
		String superClassName = "java.lang.Object";
		int classModifier = ACC_PUBLIC;
		ArrayList methodDeclarationList = new ArrayList();

		String methodName = "peek";
		int methodModifier = ACC_PUBLIC;
		Class returnType = TypeConverter.getClass("java.lang.String");
		ArrayList argumentTypeList = new ArrayList();
		MethodDeclaration methodDeclaration = new MethodDeclaration(methodName, methodModifier,
									     					 returnType, argumentTypeList);
		methodDeclarationList.add(methodDeclaration);

		methodName = "pop";
		methodModifier = ACC_PUBLIC;
		returnType = TypeConverter.getClass("java.lang.String");
		argumentTypeList = new ArrayList();
		methodDeclaration = new MethodDeclaration(methodName, methodModifier,
									     					 returnType, argumentTypeList);
		methodDeclarationList.add(methodDeclaration);

		methodName = "empty";
		methodModifier = ACC_PUBLIC;
		returnType = TypeConverter.getClass("boolean");
		argumentTypeList = new ArrayList();
		methodDeclaration = new MethodDeclaration(methodName, methodModifier,
									     					 returnType, argumentTypeList);
		methodDeclarationList.add(methodDeclaration);

		methodName = "push";
		methodModifier = ACC_PUBLIC;
		returnType = TypeConverter.getClass("java.lang.String");
		Class argumentType = TypeConverter.getClass("java.lang.String");
		argumentTypeList = new ArrayList();
		argumentTypeList.add(argumentType);
		methodDeclaration = new MethodDeclaration(methodName, methodModifier,
									     					 returnType, argumentTypeList);
		methodDeclarationList.add(methodDeclaration);

		// add a member variable

		String fieldName = "m_stack";
		methodModifier = ACC_PUBLIC;
		Class type = TypeConverter.getClass("java.util.Stack");
		FieldDeclaration fieldDeclaration = new FieldDeclaration(fieldName, methodModifier,	type);
		ArrayList fieldDeclarationList = new ArrayList();
		fieldDeclarationList.add(fieldDeclaration);

		bcf.generateClass(className, superClassName, null, classModifier,
			fieldDeclarationList, methodDeclarationList);
		Class aClass = bcf.loadClass();
		assertNotNull(aClass);

		Form ext = this.createStringStackForm();
		//Object obj = this.instantiate(aClass, ext);

		// There's no way to cast here:
		//   java.lang.Object doesn't have empty()
		// and the compiler doesn't know anything about this class:
		//  ch.unibe.piccola.bridge.StringClass;
		//
		// The way to go is use reflection on obj and then call empty by hand
		// the way it's done in the Piccola bridge.
		// assertEquals(true, (obj).empty());
	}

	private Form createStringStackForm() {
		sf.clear();
		sf.emitLookup(null, "stack");
		sf.emitProject(null, "empty");
		sf.emitEmpty(null);
		sf.emitApply(null);
		sf.emitExit(null);

		Form a = newService(sf);
        Form aBindA = a.bind("empty");

		Form ext = aBindA;

		return ext;
	}

	private Object instantiate(Class aClass, Form arg) {
		//System.err.println("Form ext: " + ext);
		// Whoever said that writing tests is fun, was wrong.
		Constructor cstr = null;
		Class par1 = null;
		try {
			par1 = Class.forName(FORM_CLASS);
		} catch (ClassNotFoundException cnfe) {cnfe.printStackTrace();}
		Class[] parameterTypes = {par1};
		try {
			cstr= aClass.getConstructor(parameterTypes);
		} catch (NoSuchMethodException nsme) {nsme.printStackTrace();}
		Object[] initArgs = {arg};

		Object obj = null;
		try {
			obj = cstr.newInstance(initArgs);
		} catch (Exception e) {e.printStackTrace();}

		return obj;
	}

}
