package ch.unibe.piccola.bytecode;

import java.lang.reflect.Method;

import org.apache.bcel.Constants;
import org.apache.bcel.generic.ClassGen;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.InstructionFactory;
import org.apache.bcel.generic.InstructionList;
import org.apache.bcel.generic.ReferenceType;
import org.apache.bcel.generic.ObjectType;
import org.apache.bcel.generic.Type;
import org.apache.bcel.generic.DUP;
import org.apache.bcel.generic.ACONST_NULL;
import org.apache.bcel.generic.PUSH;

/**
 * @author  $Id: MethodFactory.java,v 3.5 2003/01/10 13:46:27 kneubuhl Exp $
**/

public class MethodFactory implements Constants, FactoryConstants {

	private InstructionList il;

	private InstructionFactory m_factory;

	private ClassGen m_classGen;

	public MethodFactory(ClassGen cg) {
		m_classGen = cg;
		m_factory = new InstructionFactory(cg);
	}

	public ClassGen addMethod(String className, String methodName, int modifiers,
							  Class returnType, Class[] parameterTypes) {
		return init(className, methodName, modifiers, returnType, parameterTypes);
	}

	public ClassGen addMethod(String className, MethodDeclaration method) {
		String methodName = method.getName();
		int modifiers = method.getModifiers();
		Class returnType = method.getReturnType();
		Class[] parameterTypes = method.getParameterTypes();

		return init(className, methodName, modifiers, returnType, parameterTypes);
	}

	// --------------

	private ClassGen init(String className, String methodName, int modifiers,
							  Class returnType, Class[] parameterTypes)	{
		il = new InstructionList();

		MethodGen mg = this.generateMethodDeclaration(il, className, methodName, modifiers,
														returnType, parameterTypes);
		il = this.generateCallbackBlock(il, className, methodName, parameterTypes);
		il = this.generateReturnBlock(il, returnType);

		mg.setMaxStack();
		mg.setMaxLocals();
		m_classGen.addMethod(mg.getMethod());

		il.dispose();

		return m_classGen;
    }

	private MethodGen generateMethodDeclaration(InstructionList il,	String className,
												String methodName, int modifiers,
												Class returnType, Class[] parameterTypes) {

		ConstantPoolGen cp = m_classGen.getConstantPool();

		Type bcelReturnType = TypeConverter.class2bcelType(returnType);
		Type[] bcelParameterTypes = null;
		if (parameterTypes.length==0) {
			bcelParameterTypes = Type.NO_ARGS; // void a() has NO_ARGS as paramaterType !
		} else {
			bcelParameterTypes = new Type[parameterTypes.length];
			for (int i=0; i<parameterTypes.length; i++) {
		  		Class parameterType = parameterTypes[i];
				Type bcelParameterType = TypeConverter.class2bcelType(parameterType);
				bcelParameterTypes[i] = bcelParameterType;
			}
		}

		int bcelModifiers = TypeConverter.java2bcelModifiers(modifiers);
		//System.out.print("Method Modifier (before):" + bcelModifiers);
		if (bcelModifiers >= ACC_ABSTRACT) {
			// XOR ACC_ABSTRACT (interface methods are 'public abstract')
			bcelModifiers = (bcelModifiers ^ ACC_ABSTRACT);
		}
		//System.out.println("  Method Modifier (after): " + bcelModifiers);
		MethodGen mg = new MethodGen(bcelModifiers, bcelReturnType, bcelParameterTypes,
										null, methodName, className, il, cp);

		return mg;
	}

	private InstructionList generateCallbackBlock(InstructionList il, String className,
													String methodName, Class[] parameterTypes) {
		// Add this/self as first argument

		il = generateArgumentConstruction(il);
		int nextFreeLocalVariableIndex = this.calculateNextFreeLocalVariableIndex(parameterTypes);
		// 'this' takes one additional local variable
		nextFreeLocalVariableIndex++;

		// Save the Ref to Arguments in next free local variable
		il.append(m_factory.createStore(Type.OBJECT, nextFreeLocalVariableIndex));

		// add 'this' as first argument
		il.append(m_factory.createLoad(Type.OBJECT, nextFreeLocalVariableIndex));
		il.append(m_factory.createLoad(Type.OBJECT, 0));
		il.append(m_factory.createInvoke(ARGUMENTS_CLASS, "add",
											Type.VOID, new Type[] {Type.OBJECT},
										    Constants.INVOKEVIRTUAL));
		// rest of the arguments
		if (parameterTypes.length!=0) {
			// Add argument to Arguments
			for (int i=0; i<parameterTypes.length; i++) {
				il.append(m_factory.createLoad(Type.OBJECT, nextFreeLocalVariableIndex));

				Class parameterType = parameterTypes[i];
				if (!(parameterType.isPrimitive())) {
					ReferenceType bcelParameterType = (ReferenceType) TypeConverter.class2bcelType(parameterType);
					// Load non-primitive Object on Stack, load_0 is 'this', so we start at 1
					il.append(m_factory.createLoad(bcelParameterType, (i+1)));
				} else {
					il = convertPrimitiveToObject(il, parameterType, (i+1));
				}
				il.append(m_factory.createInvoke(ARGUMENTS_CLASS, "add",
									    	   Type.VOID, new Type[] {Type.OBJECT},
										       Constants.INVOKEVIRTUAL));
			}
		}

		// Field Access
		il = this.generateServiceHandlerCall(il, className, methodName);

		// Load Arguments-Ref on to Stack
		il.append(m_factory.createLoad(Type.OBJECT, nextFreeLocalVariableIndex));

		il.append(m_factory.createInvoke(FORM_CLASS, "callback",
				       Type.OBJECT, new Type[] {new ObjectType(ARGUMENTS_CLASS)},
				       Constants.INVOKEVIRTUAL));
		return il;
	}

	private InstructionList generateReturnBlock(InstructionList il, Class returnType) {
		if (returnType.getName() == "void") {
			il.append(m_factory.createPop(1));
		} else {
			if(!(returnType.isPrimitive())) {
				ReferenceType classToCastTo = (ReferenceType) TypeConverter.class2bcelType(returnType);
				il.append(m_factory.createCheckCast(classToCastTo)); // Check cast ..
			} else {
				il = convertObjectToPrimitive(il, returnType);
			}
		}
		il.append(m_factory.createReturn(TypeConverter.class2bcelType(returnType)));
		return il;
	}

 	// ----------------------

	private InstructionList generateArgumentConstruction(InstructionList il) {
		/* Create Arguments */
		// Call new, the standard constructor
		il.append(m_factory.createNew(ARGUMENTS_CLASS));
		// duplicate THIS
		il.append(new DUP());
		// Invoke Constructor
		il.append(m_factory.createInvoke(ARGUMENTS_CLASS, "<init>",
											Type.VOID, Type.NO_ARGS,
											Constants.INVOKESPECIAL));
		return il;
	}

	private InstructionList generateServiceHandlerCall(InstructionList il, String className, String methodName) {
		ConstantPoolGen cp = m_classGen.getConstantPool();
		il.append(m_factory.createThis());
		il.append(new PUSH(cp, methodName));
		il.append(m_factory.createInvoke(className, "getService",
				       new ObjectType(FORM_CLASS),
				       new Type[] {new ObjectType("java.lang.String")},
				       Constants.INVOKEVIRTUAL));
		return il;
	}

    // --- Helper Functions ---

	private int calculateNextFreeLocalVariableIndex(Class[] parameterTypes) {
		int maxArgs = 1;
		for (int i=0; i<parameterTypes.length; i++) {
			Class parameterType = parameterTypes[i];
			// 'double' and 'long' take up two local variables
			if ((parameterType.getName() == "double") || (parameterType.getName() == "long")) {
				maxArgs += 2;
			} else {
				maxArgs += 1;
			}
		}
		return maxArgs;
	}

	private InstructionList convertPrimitiveToObject(InstructionList il, Class parameterType, int parameterTypeIndex) {
		Type bcelPrimitiveParameterType = TypeConverter.class2bcelType(parameterType);

		// Arrgh, primitive datatype -> Create an Object of it
		String ObjectClassName = TypeConverter.getClassForPrimitiveType(parameterType);
		// Call new, the standard constructor
		il.append(m_factory.createNew(ObjectClassName));
		// duplicate THIS
		il.append(new DUP());
		// Load the parameter on stack
		il.append(m_factory.createLoad(bcelPrimitiveParameterType, parameterTypeIndex));
		// Invoke Constructor
		il.append(m_factory.createInvoke(ObjectClassName, "<init>",
										Type.VOID, new Type[] {bcelPrimitiveParameterType},
									   	Constants.INVOKESPECIAL));
		return il;
	}

	private InstructionList convertObjectToPrimitive(InstructionList il, Class returnType) {
		Type bcelPrimitiveReturnType = TypeConverter.class2bcelType(returnType);

		// Piccola returns the primitive type wrapped as an object
		ObjectType classToCastTo = new ObjectType(TypeConverter.getClassForPrimitiveType(returnType));
		il.append(m_factory.createCheckCast(classToCastTo));
		// TypeConvert it to primitive type  by invoking <type>Value()
		String methodName =	returnType.getName()+"Value";
		String returnTypeClassName = TypeConverter.getClassForPrimitiveType(returnType);
		// Invoke <primType>Value() method
		il.append(m_factory.createInvoke(returnTypeClassName, methodName,
									bcelPrimitiveReturnType, Type.NO_ARGS,
									Constants.INVOKEVIRTUAL));
		return il;
	}
}
