package ch.unibe.piccola.bytecode;

import ch.unibe.piccola.PiccolaException;

import java.lang.reflect.Modifier;
import java.util.List;

import org.apache.bcel.Constants;
import org.apache.bcel.generic.ObjectType;
import org.apache.bcel.generic.ArrayType;
import org.apache.bcel.generic.Type;

/**
 * @author  $Id: TypeConverter.java,v 3.5 2003/01/10 13:46:28 kneubuhl Exp $
**/

public class TypeConverter {

	public static Type class2bcelType(Class aClass) {
		Type bcelType = null;
		if (aClass.isPrimitive()) {
			if (aClass == void.class) {
				bcelType = Type.VOID;
			} else if (aClass == boolean.class) {
				bcelType = Type.BOOLEAN;
			} else if (aClass == int.class) {
				bcelType = Type.INT;
			} else if (aClass == short.class) {
				bcelType = Type.SHORT;
			} else if (aClass == byte.class) {
				bcelType = Type.BYTE;
			} else if (aClass == long.class) {
				bcelType = Type.LONG;
			} else if (aClass == double.class) {
				bcelType = Type.DOUBLE;
			} else if (aClass == float.class) {
				bcelType = Type.FLOAT;
			} else if (aClass == char.class) {
				bcelType = Type.CHAR;
			}
		} else if (aClass.isArray()) {
			int dimension = 0;
			do {
				aClass = aClass.getComponentType();
				++dimension;
			} while (aClass.isArray());
			Type ArrayType = class2bcelType(aClass);
			bcelType = new ArrayType(ArrayType, dimension);	
		} else {
			bcelType = new ObjectType(aClass.getName());
		}  
		return bcelType;
	}

	public static String getClassForPrimitiveType(Class aClass) {
		String className = "java.lang.";
		if (aClass == boolean.class) {
			className += "Boolean";
		} else if (aClass == int.class) {
			className += "Integer";
		} else if (aClass == short.class) {
			className += "Short";
		} else if (aClass == byte.class) {
			className += "Byte";
		} else if (aClass == long.class) {
			className += "Long";
		} else if (aClass == double.class) {
			className += "Double";
		} else if (aClass == float.class) {
			className += "Float";
		} else if (aClass == char.class) {
			className += "Character";
		} else {
			throw new PiccolaException("Bridge: Not a primitive Type: " + aClass.getName());
		}
		return className;
	}

	public static Class getClass(String className) {
		Class aClass;
		if (className.equals("void")) {
			aClass = void.class;
		} else if (className.equals("boolean")) {
			aClass = boolean.class;
		} else if (className.equals("int")) {
			aClass = int.class;
		} else if (className.equals("short")) {
			aClass = short.class;
		} else if (className.equals("byte")) {
			aClass = byte.class;
		} else if (className.equals("long")) {
			aClass = long.class;
		} else if (className.equals("double")) {
			aClass = double.class;
		} else if (className.equals("float")) {
			aClass = float.class;
		} else if (className.equals("char")) {
			aClass = char.class;
		} else {
			try {
      			aClass = Class.forName(className);
    		}
	    	catch(ClassNotFoundException e) {
				throw new PiccolaException("Bridge: Class `" + className + "' not found");
			}
		}
		return aClass;
	}

	public static int java2bcelModifiers(int modifier) {
		// ACC_SUPER has no equivalent in java.lang.reflect.Modifier
		short accessModifier = 0;
		if (Modifier.isPublic(modifier)) {
			accessModifier |= Constants.ACC_PUBLIC;
		}
		if (Modifier.isPrivate(modifier)) {
			accessModifier |= Constants.ACC_PRIVATE;
		}
		if (Modifier.isProtected(modifier)) {
			accessModifier |= Constants.ACC_PROTECTED;
		}
		if (Modifier.isStatic(modifier)) {
			accessModifier |= Constants.ACC_STATIC;
		}

		if (Modifier.isFinal(modifier)) {
			accessModifier |= Constants.ACC_FINAL;
		}
		if (Modifier.isSynchronized(modifier)) {
			accessModifier |= Constants.ACC_SYNCHRONIZED;
		}
		if (Modifier.isVolatile(modifier)) {
			accessModifier |= Constants.ACC_VOLATILE;
		}
		if (Modifier.isStatic(modifier)) {
			accessModifier |= Constants.ACC_STATIC;
		}

		if (Modifier.isNative(modifier)) {
			accessModifier |= Constants.ACC_NATIVE;
		}
		if (Modifier.isInterface(modifier)) {
			accessModifier |= Constants.ACC_INTERFACE;
		}
		if (Modifier.isAbstract(modifier)) {
			accessModifier |= Constants.ACC_ABSTRACT;
		}
		if (Modifier.isStrict(modifier)) {
			accessModifier |= Constants.ACC_STRICT;
		}

		return accessModifier;
	}
/*
	public static String[] convertListToStringArray(List list) {
		System.err.println(list.getClass());
		System.err.println(list.get(0).getClass());
		return (String[]) list.toArray(new String[0]);
	}
*/
}
