package ch.unibe.piccola.code;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Represents an argument list for a method call.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Dec 09, 2002)
**/
public class Arguments {

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Creates an empty argument list.
    **/
    public Arguments()
    {
        m_args = new ArrayList();
    }

    /**
     * Creates an argument list with one argument.
     *
     * @param  arg1 the argument.
    **/
    public Arguments(Object arg1)
    {
        this();
        add(arg1);
    }

    /**
     * Creates an argument list with two arguments.
     *
     * @param  arg1 the first argument.
     * @param  arg1 the second argument.
    **/
    public Arguments(Object arg1, Object arg2)
    {
        this(arg1);
        add(arg2);
    }

    /**
     * Creates an argument list with three arguments.
     *
     * @param  arg1 the first argument.
     * @param  arg1 the second argument.
     * @param  arg1 the third argument.
    **/
    public Arguments(Object arg1, Object arg2, Object arg3)
    {
        this(arg1, arg2);
        add(arg3);
    }

    /**
     * Adds an argument to this argument list.
     *
     * @param  arg the argument to be added to this list.
    **/
    public void add(Object arg)
    {
        m_args.add(arg);
    }

    /**
     * Return the argument types of this argument list in an array. The
     * type of the argument <code>null</code> is <code>null</code>, the
     * type of any other argument <code>arg</code> is
     * <code>arg.getClass()</code>.
     *
     * @return an array containing the types of the arguments represented by
     *         this argument list.
    **/
    public Class[] argumentTypes()
    {
        Class[] result = new Class[m_args.size()];
        for (int i = 0; i < m_args.size(); ++i) {
            result[i] = null == m_args.get(i) ?
                null : m_args.get(i).getClass();
        }
        return result;
    }

    /**
     * Compares this argument list to the specified object. Returns true if,
     * and only if the object is an instance of this class and contains
     * the same arguments (equality) in the same order.
     *
     * @param  object the object to be compared to this argument list.
     * @return <code>true</code>, if object is equal to this argument list,
     *         <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (object instanceof Arguments) {
            return ((Arguments) object).m_args.equals(m_args);
        }
        return false;
    }

    /**
     * Returns the argument at the specified position in this argument list.
     *
     * @param  index zero-based index of argument to return.
     * @return the argument at the specified position in this argument list.
     * @throws IndexOutOfBoundsException if the index is out of range
     *         (<code>index < 0 || index >= size()</code>).
    **/
    public Object get(int index)
    {
        return m_args.get(index);
    }

    /**
     * Returns a hash code value for this argument list.
     *
     * @return a hash code value for this argument list.
    **/
    public int hashCode()
    {
        return m_args.hashCode();
    }

    /**
     * Returns the number of elements in this list. If this list contains
     * more than <code>Integer.MAX_VALUE</code> elements, returns
     * <code>Integer.MAX_VALUE</code>.
     *
     * @return the number of elements in this list.
    **/
    public int size()
    {
        return m_args.size();
    }

    /**
     * Returns an array containing all of the arguments in this list in proper
     * sequence.
     *
     * @return an array containing all of the arguments in this list in proper
     *         sequence.
    **/
    public Object[] toArray()
    {
        return m_args.toArray();
    }

    /**
     * Returns a string representation of this argument list.
     *
     * @return a string representation of this argument list.
    **/
    public String toString()
    {
        String result = "(";
        if (0 < m_args.size()) {
            result += m_args.get(0);
        }
        for (int i = 1; i < m_args.size(); ++i) {
            result += ", " + m_args.get(i);
        }
        return result + ")";
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private ArrayList m_args;

}