package ch.unibe.piccola.code;

/**
 * The opcode interface of the Piccola virtual machine. The Piccola opcodes use
 * this interface to invoke their specific action on the virtual machine.
 *
 * @author   Stefan Kneubuehl
 * @version  3.6c (May 11, 2003)
**/
public interface IVM
{

    public void opApply();

    /**
     * Executes a <b>bind</b> opcode on this virtual machine. The bind opcode
     * has a parameter.
     *
     * <table border="1" cellspacing="0" style="empty-cells:show">
     *  <tr><td><b>stack</b></td><td><b>before</b></td><td><b>after</b></td></tr>
     *  <tr><td>value </td><td>F     </td><td>(label = F)</td></tr>
     *  <tr><td>static</td><td>      </td><td>           </td></tr>
     *  <tr><td>code  </td><td>      </td><td>           </td></tr>
     * </table>
     *
     * @param  label the label parameter for this <b>bind</b> opcode.
    **/
    public void opBind(String label);

    public void opEmpty();

    public void opExit();

    public void opExtend();

    public void opHide(String label);

    public void opJavaCall(JavaOp javaOp, Object target);

    public void opJavaObject(Object object);

    public void opLookup(String label);

    public void opMeta();

    public void opPopContext();

    public void opPopValue();

    public void opProject(String label);

    public void opPushContext();

    public void opReceive(Object channel);

    public void opSend(Object channel);

    public void opService(Code code);

    public void opRoot();

    public void opVariable();

}

