package ch.unibe.piccola.syntax;

import java.util.Arrays;
import ch.unibe.piccola.source.SourceRef;

/**
 * Represents a compound binding syntax node.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
public class CompoundBinding extends TwoChildrenNode
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Accepts a node visitor.
     *
     * @param  visitor the node visitor to be accepted.
     *
     * @return the result of the visitor's visitCompoundBinding method.
    **/
    public Object accept(NodeVisitor visitor)
    {
        return visitor.visitCompoundBinding(this);
    }

    /**
     * Compares this compound binding node with the specified object.
     *
     * @param  object the object to be compared with this node.
     *
     * @return <code>true</code> if this compound binding node is equal to
     *         object, <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (!super.equals(object)) {
            return false;
        }
        CompoundBinding other = (CompoundBinding) object;
        return m_fixedPoint == other.m_fixedPoint &&
               Arrays.equals(m_labels, other.m_labels);
    }

    /**
     * Returns the labels of this compound binding node.
     *
     * @return   the labels of this compound binding node.
    **/
    public String[] getLabels()
    {
        return m_labels;
    }

    /**
     * Tests if this compound binding node is a fixed point.
     *
     * @return <code>true</code> if this CompoundBinding node is a fixed point,
     *         <code>false</code> otherwise.
    **/
    public boolean isFixedPoint()
    {
        return m_fixedPoint;
    }

    /**
     * Returns a debug string representation of this compound binding node.
     *
     * @return a debug string representation of this xompound binding node.
    **/
    public String toString()
    {
        return "CompoundBinding(" + labelsString(m_fixedPoint, m_labels) +
            ", " + getLeftChild() + ", " + getRightChild() + ")";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    CompoundBinding(SourceRef sourceRef, Node leftChild, Node rightChild,
                    String[] labels, boolean fixedPoint)
    {
        super(sourceRef, leftChild, rightChild);
        m_labels = labels;
        m_fixedPoint = fixedPoint;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Node newInstance(Node leftChild, Node rightChild)
    {
        return new CompoundBinding(
            getSourceRef(), leftChild, rightChild, m_labels, m_fixedPoint
        );
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final String[] m_labels;

    private final boolean m_fixedPoint;

}