package ch.unibe.piccola.syntax;

import ch.unibe.piccola.source.SourceRef;

/**
 * Represents an infix application syntax node.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
public class InfixApplication extends TwoChildrenNode
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Accepts a node visitor.
     *
     * @param  visitor the node visitor to be accepted.
     *
     * @return the result of the visitor's visitInfixApplication method.
    **/
    public Object accept(NodeVisitor visitor)
    {
        return visitor.visitInfixApplication(this);
    }

    /**
     * Compares this InfixApplication node with the specified Object.
     *
     * @param  object the object to be compared with this node.
     *
     * @return <code>true</code> if this infix application node is equal to
     *         object, <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (!super.equals(object)) {
            return false;
        }
        InfixApplication other = (InfixApplication) object;
        return m_label.equals(other.m_label);
    }

    /**
     * Returns the operator of this infix application node.
     *
     * @return the operator of this infix application node.
    **/
    public String getLabel()
    {
        return m_label;
    }

    /**
     * Returns a debug string representation of this infix application node.
     *
     * @return a debug string representation of this infix application node.
    **/
    public String toString()
    {
        return "InfixApplication(" + m_label + ", " + getLeftChild() +
            ", " + getRightChild() + ")";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    InfixApplication(SourceRef sourceRef, Node leftChild, Node rightChild,
                     String label)
    {
        super(sourceRef, leftChild, rightChild);
        m_label = label;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Node newInstance(Node leftChild, Node rightChild)
    {
        return new InfixApplication(
            getSourceRef(), leftChild, rightChild, m_label
        );
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final String m_label;

}