package ch.unibe.piccola.syntax;

import ch.unibe.piccola.source.SourceRef;

/**
 * Represents a syntax node with one child node and an associated label. The
 * following node types have one child and a label:
 * <ul>
 *   <li>Binding
 *   <li>FixedPoint
 *   <li>Hide
 *   <li>PrefixApplication
 *   <li>Projection
 * </ul>
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
abstract class LabelledOneChildNode extends OneChildNode
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Compares this node with the specified object.
     *
     * @param  object the object to be compared with this node.
     *
     * @return <code>true</code> if this node is equal to object,
     *         <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (!super.equals(object)) {
            return false;
        }
        LabelledOneChildNode other = (LabelledOneChildNode) object;
        return m_label.equals(other.m_label);
    }

    /**
     * Returns the label associated with this node.
     *
     * @return the label associated with this node.
    **/
    public final String getLabel()
    {
        return m_label;
    }

    /**
     * Returns a debug string representation of this node.
     *
     * @return a debug string representation of this node.
    **/
    public String toString()
    {
        return "(" + getLabel() + ", " + getChild() + ")";
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected LabelledOneChildNode(SourceRef sourceRef, Node child,
                                   String label)
    {
        super(sourceRef, child);
        m_label = label;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final String m_label;

}