package ch.unibe.piccola.syntax;

import java.util.Arrays;
import ch.unibe.piccola.source.SourceRef;

/**
 * Represents a named abstraction syntax node.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/

public class NamedAbstraction extends TwoChildrenNode
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Accepts a node visitor.
     *
     * @param  visitor the node visitor to be accepted.
     *
     * @return the result of the visitor's visitNamedAbstraction method.
    **/
    public Object accept(NodeVisitor visitor)
    {
        return visitor.visitNamedAbstraction(this);
    }

    /**
     * Compares this named abstraction node with the specified object.
     *
     * @param  object the object to be compared with this node.
     *
     * @return <code>true</code> if this named abstraction node is equal to
     *         object, <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (!super.equals(object)) {
            return false;
        }
        NamedAbstraction other = (NamedAbstraction) object;
        return m_fixedPoint == other.m_fixedPoint &&
               Arrays.equals(m_labels, other.m_labels) &&
               Arrays.equals(m_parameters, other.m_parameters);
    }

    /**
     * Returns the labels of this named abstraction node.
     *
     * @return the labels of this named abstraction node.
    **/
    public String[] getLabels()
    {
        return m_labels;
    }

    /**
     * Returns the parameters of this named abstraction node.
     *
     * @return the parameters of this named abstraction node.
    **/
    public String[] getParameters()
    {
        return m_parameters;
    }

    /**
     * Tests if this named abstraction node is a fixed point.
     *
     * @return <code>true</code> if this named abstraction node is a fixed
     *         point, <code>false</code> otherwise.
    **/
    public boolean isFixedPoint()
    {
        return m_fixedPoint;
    }

    /**
     * Returns a debug string representation of this named abstraction node.
     *
     * @return a debug string representation of this named abstraction node.
    **/
    public String toString()
    {
        return "NamedAbstraction(" + labelsString(m_fixedPoint, m_labels) +
            ", " + parametersString(m_parameters) + ", " + getLeftChild() +
            ", " + getRightChild() + ")";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    NamedAbstraction(SourceRef sourceRef, Node leftChild, Node rightChild,
                     String[] labels, String[] parameters, boolean fixedPoint)
    {
        super(sourceRef, leftChild, rightChild);
        m_labels = labels;
        m_parameters = parameters;
        m_fixedPoint = fixedPoint;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Node newInstance(Node leftChild, Node rightChild)
    {
        return new NamedAbstraction(getSourceRef(), leftChild, rightChild,
            m_labels, m_parameters, m_fixedPoint);
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final String[] m_labels;

    private final String[] m_parameters;

    private final boolean m_fixedPoint;

}