package ch.unibe.piccola.syntax;

import ch.unibe.piccola.syntax.*;

/**
 * Base class for node visitors. The visitor implements the following default
 * behaviour:
 * <ul>
 *   <li>For leaf nodes: <i>visit[n] = n</i>
 *   <li>For nodes with one child: <i>visit[n(c)] = n(visit[c])</i>
 *   <li>For nodes with two children: <i>visit[n(c, d)] = n(visit[c],
 *       visit[d])</i>
 * </ul>
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
public abstract class NodeVisitor
{

    // ------------------------------------------------------------------------
    // protected static
    // ------------------------------------------------------------------------

    protected static final SyntaxFactory sf = SyntaxFactory.INSTANCE;

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public Object visitAbstraction(Abstraction node)
    {
        return defaultAction(node);
    }

    public Object visitApplication(Application node)
    {
        return defaultAction(node);
    }

    public Object visitBinding(Binding node)
    {
        return defaultAction(node);
    }

    public Object visitCollection(Collection node)
    {
        return defaultAction(node);
    }

    public Object visitCompoundBinding(CompoundBinding node)
    {
        return defaultAction(node);
    }

    public Object visitConstant(Constant node)
    {
        return defaultAction(node);
    }

    public Object visitCurriedAbstraction(CurriedAbstraction node)
    {
        return defaultAction(node);
    }

    public Object visitEmpty(Empty node)
    {
        return defaultAction(node);
    }

    public Object visitExtension(Extension node)
    {
        return defaultAction(node);
    }

    public Object visitFixedPoint(FixedPoint node)
    {
        return defaultAction(node);
    }

    public Object visitHide(Hide node)
    {
        return defaultAction(node);
    }

    public Object visitInfixApplication(InfixApplication node)
    {
        return defaultAction(node);
    }

    public Object visitLabel(Label node)
    {
        return defaultAction(node);
    }

    public Object visitNamedAbstraction(NamedAbstraction node)
    {
        return defaultAction(node);
    }

    public Object visitPrefixApplication(PrefixApplication node)
    {
        return defaultAction(node);
    }

    public Object visitProjection(Projection node)
    {
        return defaultAction(node);
    }

    public Object visitQuote(Quote node)
    {
        return defaultAction(node);
    }

    public Object visitRoot(Root node)
    {
        return defaultAction(node);
    }

    public Object visitSandbox(Sandbox node)
    {
        return defaultAction(node);
    }

    public Object defaultAction(Node node)
    {
        return node.transform(this);
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected NodeVisitor()
    {
    }

    protected Node visit(Node node)
    {
        return (Node) node.accept(this);
    }

}