package ch.unibe.piccola.syntax;

import ch.unibe.piccola.source.SourceRef;

/**
 * Represents a syntax node with one child node. The following node types
 * have one child:
 * <ul>
 *   <li>Abstraction
 *   <li>CurriedAbstraction
 *   <li>LabelledOneChildNode
 * </ul>
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
abstract class OneChildNode extends Node
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Compares this node with the specified object.
     *
     * @return <code>true</code> if this node is equal to object,
     *         <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (!super.equals(object)) {
            return false;
        }
        OneChildNode other = (OneChildNode) object;
        return m_child.equals(other.m_child);
    }

    /**
     * Returns the child of this node.
     *
     * @return the child of this node.
    **/
    public Node getChild()
    {
        return m_child;
    }

    public Node transform(NodeVisitor visitor)
    {
        Node child = (Node) m_child.accept(visitor);
        return (child == m_child) ? this : newInstance(child);
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected OneChildNode(SourceRef sourceRef, Node child)
    {
        super(sourceRef);
        m_child = child;
    }

    protected abstract Node newInstance(Node child);

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Node m_child;

}