package ch.unibe.piccola.type;

/**
 * Represents a type constraint.
 *
 * @author    Stefan Kneubuehl
 * @version   3.4c (Jan 04, 2003)
**/
public class Constraint
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Creates a new constraint.
    **/
    public Constraint(Type provided, Type required)
    {
        m_provided = provided;
        m_required = required;
    }

    public Type getProvided()
    {
        return m_provided;
    }

    public Type getRequired()
    {
        return m_required;
    }

    public boolean isLabelConstraint(String x)
    {
        return m_provided == Type.ROOT & m_required instanceof TypeLabel &&
            ((TypeLabel) m_required).getLabel().equals(x) &&
            ((TypeLabel) m_required).getType() instanceof TypeVariable;
    }

    public Type getLabelType(String x)
    {
        if (isLabelConstraint(x)) {
            return ((TypeLabel) m_required).getType();
        }
        return Type.EMPTY;
    }

    /**
     * Checks whether the specified type variable occurs in this constraint.
    **/
    public boolean occurs(TypeVariable var)
    {
        return m_provided.occurs(var) || m_required.occurs(var);
    }

    public Constraint subst(TypeVariable var, Type replace)
    {
        return new Constraint(m_provided.subst(var, replace),
                              m_required.subst(var, replace));
    }

    /**
     * Returns a string representation of this constraint.
    **/
    public String toString()
    {
        return m_provided.toString() + " '-> " + m_required.toString();
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Type m_provided;

    private final Type m_required;

}