package ch.unibe.piccola.type;

import java.util.ArrayList;

/**
 * Represents a set of type constraints.
 *
 * @author    Stefan Kneubuehl
 * @version   3.4c (Jan 04, 2003)
**/
public class Constraints
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Creates a new constraints list.
    **/
    public Constraints()
    {
        m_list = new ArrayList();
    }

    /**
     * Adds the specified constraint provided '-> required to these constraints.
     * Resolves all constraint equivalences.
    **/
    public void add(Type provided, Type required)
    {
        m_list.add(new Constraint(provided, required));
    }

    /**
     * Adds the specified constraints to this list.
    **/
    public void addAll(Constraints c)
    {
        for (int i = 0; i < c.m_list.size(); ++i) {
            m_list.add(c.get(i));
        }
    }

    public boolean isResolvable(ContractualType ct)
    {
        return findSubstIndex(ct) != -1;
    }

    public boolean isEmpty()
    {
        return m_list.isEmpty();
    }

    /**
     * Simplifies these constraints by applying equivalences.
    **/
    public void normalize()
    {
        ArrayList list = new ArrayList();
        for (int i = 0; i < m_list.size(); ++i) {
            get(i).getRequired().simplifyConstraint(list, get(i).getProvided());
        }
        m_list = list;
    }

    public Type opLookupLabel(String x)
    {
        Type result = Type.EMPTY;
        Type r;
        for (int i = 0; i < m_list.size(); ++i) {
            result = result.and(get(i).getLabelType(x));
        }
        return result;
    }

    public void opRemoveLabel(String x)
    {
        for (int i = 0; i < m_list.size(); ++i) {
            if (get(i).isLabelConstraint(x)) {
                m_list.remove(get(i));
                --i;
            }
        }
    }

    public void resolve(ContractualType ct)
    {
        int i;
        Type p, r;
        Constraint c;

        i = findSubstIndex(ct);
        while (-1 != i) {
            c = get(i);
            m_list.remove(c);
            if (bindLeft) {
                doSubst(ct, (TypeVariable) c.getProvided(), c.getRequired());
            }
            else {
                doSubst(ct, (TypeVariable) c.getRequired(), c.getProvided());
            }
            i = findSubstIndex(ct);
        }
    }

    /**
     * Substitutes the specified type variable with the specified
     * type expression in these constraints.
    **/
    public void subst(TypeVariable var, Type replace)
    {
        ArrayList list = new ArrayList();
        for (int i = 0; i < m_list.size(); ++i) {
            list.add(get(i).subst(var, replace));
        }
        m_list = list;
    }

    public String toString()
    {
        String result = "";
        if (0 < m_list.size()) {
            result = m_list.get(0).toString();
        }
        for (int i = 1; i < m_list.size(); ++i) {
            result = result + ", " + m_list.get(i);
        }
        return result;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private boolean occurs(TypeVariable var, int excludedIndex)
    {
        for (int i = 0; i < m_list.size(); ++i) {
            if (i != excludedIndex && get(i).occurs(var)) { return true; }
        }
        return false;
    }

    private void doSubst(ContractualType ct, TypeVariable var, Type replace)
    {
        ct.subst((TypeVariable) var.reverse(), replace);
        if (replace instanceof TypeVariable) {
            ct.subst(var, replace.reverse());
        }
    }

    private Constraint get(int index)
    {
        return (Constraint) m_list.get(index);
    }

    /**
     * Finds the index of a constraint that can be bound by a bind rule.
     * Returns -1 if no such constraint is available in this list.
    **/
    private int findSubstIndex(ContractualType ct)
    {
        Type p, r;
        for (int i = m_list.size() - 1; i >= 0; --i) {
            p = get(i).getProvided();
            r = get(i).getRequired();
            if (isCandidate(ct, p, i)) {
                bindLeft = true;
                return i;
            }
            if (isCandidate(ct, r, i)) {
                bindLeft = false;
                return i;
            }
        }
        return -1;
    }

    private boolean isCandidate(ContractualType ct, Type t, int excludedIndex)
    {
        if (t instanceof TypeVariableNormal) {
            TypeVariable v = (TypeVariable) t;
            return !occurs(v, excludedIndex) && ! ct.getProvided().occurs(v);
        }
        return false;
    }

    private ArrayList m_list;

    private boolean bindLeft;

}