package ch.unibe.piccola.type;

import ch.unibe.piccola.syntax.*;

/**
 * @author   Stefan Kneubuehl
 * @version  3.4c (Jan 04, 2002)
**/
public class TypeInference extends SimplifiedNodeVisitor
{

    // ------------------------------------------------------------------------
    // public static
    // ------------------------------------------------------------------------

    public static ContractualType apply(Node node)
    {
        TypeInference t = new TypeInference();
        try {
            Type.resetVariables(); // reset variable counter
            ContractualType ct = t.inferType(node);
            ct.typeCheck();
            return ct;
        }
        catch (TypeException ex) {
            System.out.println("TYPE ERROR " + ex.getMessage() + " at:");
            System.out.println(t.m_currNode.getSourceRef());
        }
        return null;
    }

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Applies the type rule 'Abstract'.
    **/
    public Object visitAbstraction(Abstraction node)
    {
        return inferType(node.getChild()).service(node.getParameter());
    }

    /**
     * Applies the type rule 'Application'.
    **/
    public Object visitApplication(Application node)
    {
        return inferType(node.getLeftChild()).apply(
            inferType(node.getRightChild())
        );
    }

    /**
     * Applies the type rule 'Label'.
    **/
    public Object visitBinding(Binding node)
    {
        return inferType(node.getChild()).label(node.getLabel());
    }

    public Object visitConstant(Constant node)
    {
        throw new RuntimeException("not implmenented yet");
    }

    /**
     * Applies the type rule 'Empty'.
    **/
    public Object visitEmpty(Empty node)
    {
        return ContractualType.newEmpty();
    }

    /**
     * Applies the type rule 'Extend'.
    **/
    public Object visitExtension(Extension node)
    {
        return inferType(node.getLeftChild()).extend(
            inferType(node.getRightChild())
        );
    }

    public Object visitFixedPoint(FixedPoint node)
    {
        return inferType(node.getChild());
    }

    public Object visitHide(Hide node)
    {
        return inferType(node.getChild()).restrict(node.getLabel());
    }

    /**
     * Applies the type rule 'Lookup'.
    **/
    public Object visitLabel(Label node)
    {
        return ContractualType.newLookup(node.getLabel());
    }

    /**
     * The projection 'F.x' is a abbreviation of F;x. Thus, this
     * method applies the type rule 'Close'.
    **/
    public Object visitProjection(Projection node)
    {
        return inferType(node.getChild()).close(ContractualType.newLookup(node.getLabel()));
    }

    public Object visitRoot(Root node)
    {
        return ContractualType.newRoot();
    }

    /**
     * Applies the type rule 'Close'.
    **/
    public Object visitSandbox(Sandbox node)
    {
        return inferType(node.getLeftChild()).close(inferType(node.getRightChild()));
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private Node m_currNode;

    private TypeInference()
    {
    }

    private ContractualType inferType(Node node)
    {
        m_currNode = node;
        return (ContractualType) node.accept(this);
    }

}