package ch.unibe.piccola.type;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import ch.unibe.piccola.compiler.Simplifier;
import ch.unibe.piccola.parser.Parser;
import ch.unibe.piccola.parser.ParserException;
import ch.unibe.piccola.parser.ParserFactory;
import ch.unibe.piccola.source.StringSourceCode;
import ch.unibe.piccola.syntax.Node;

/**
 * @author    Stefan Kneubuehl
 * @version   3.4c (Jan 05, 2001)
**/
public class TypeInferenceTest extends TestCase {

    // ------------------------------------------------------------------------
    // test suite
    // ------------------------------------------------------------------------

    public TypeInferenceTest(String name)
    {
        super(name);
    }

    public static TestSuite suite()
    {
        return new TestSuite(TypeInferenceTest.class);
    }

    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(suite());
    }

    // ------------------------------------------------------------------------
    // test environment
    // ------------------------------------------------------------------------

    public void setUp()
    {
        parser = ParserFactory.INSTANCE.newJavaCCParser();
    }

    private Parser parser;

    private Node parse(String code)
    {
        try {
            return parser.parse(new StringSourceCode(code));
        }
        catch (ParserException ex) {
            fail("parsing failed: while parsing \"" + code + "\" " + ex);
        }
        catch (Throwable ex) {
            fail("Unexpected exception: " + ex);
        }
        return null;
    }

    public ContractualType inferType(String code)
    {
        return TypeInference.apply(Simplifier.apply(parse(code)));
    }

    public void assertType(String code, String type)
    {
//        System.out.println("TYPE OF " + code);
        ContractualType t = inferType(code);
//        System.out.println("IS " + t + "\n");
        assertTrue(t != null);
        assertEquals(type, t.toString());
    }

    // ------------------------------------------------------------------------
    // tests
    // ------------------------------------------------------------------------

    public void testExample01()
    {
        assertType("x = y", "x:A+ | ROOT '-> y:A-");
    }

    public void testExample02()
    {
        assertType("x, y", "(A+,B+) | ROOT '-> (x:A-^y:B-)");
    }

    public void testExample03()
    {
        assertType("\\x: x", "(A-->A+)");
    }

    public void testExample04()
    {
        assertType("\\x: y", "(()->A+) | ROOT '-> y:A-");
    }

    public void testExample05()
    {
        assertType("root = x, y", "B+ | ROOT '-> x:y:B-");
    }

    public void testExample06()
    {
        assertType("x y", "C+ | ROOT '-> (x:(B+->C-)^y:B-)");
    }

    public void testExample07()
    {
        assertType("\\x: a = x", "(A-->a:A+)");
    }

    public void testExample08()
    {
        assertType("\\x: x.y", "(y:B-->B+)");
    }

    public void testExample09()
    {
        assertType("\\x: \\y: (x, y)", "(A-->(B-->(A+,B+)))");
    }

    public void testExample10()
    {
        assertType("\\x: \\y: x.y", "(y:B-->(()->B+))");
    }

    public void testExample11()
    {
        assertType("(\\x: x) (\\x: x)", "(C-->C+)");
    }

    public void testExample12()
    {
        assertType("(\\x: x.b) (b=())", "()");
    }

    public void testExample13()
    {
        assertType("(\\x: x.b) z", "E+ | ROOT '-> z:b:E-");
    }

    public void testQuote()
    {
        assertType("'x = (), x", "()");
    }

    public void testRoot1()
    {
        assertType("root = root, x", "A+ | ROOT '-> x:A-");
    }

    public void testRoot2()
    {
        assertType("root = (), b = root", "b:()");
    }

    public void testRoot3()
    {
        assertType("b = root", "b:ROOT");
    }

    public void testRoot4()
    {
        assertType("root = (x = ()), (root = (root, y = ()), root)", "(x:(),y:())" );
        assertType("root = (x = ()), ('y = (), root)", "(x:(),y:())" );
    }

    public void testRoot5()
    {
        assertType("root = (x = ()), ('y = (), x)", "()" );
    }

    public void testRoot6()
    {
        assertType("x = newChannel(), x.send()", "(x:C+,F+) | ROOT '-> newChannel:(()->(C-^send:(()->F-)))");
    }

    public void testRoot7()
    {
        assertType("x: (), x()", "x:(()->())");
    }

}