package ch.unibe.piccola.type;

import java.util.ArrayList;

/**
 * Represents the service type 'P->R' or 'R->P'.
 *
 * @author    Stefan Kneubuehl
 * @version   3.4c (Jan 04, 2003)
**/
public class TypeService extends Type
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public boolean equals(Object object)
    {
        if (object instanceof TypeService) {
            TypeService other = (TypeService) object;
            return m_from.equals(other.m_from) &&
                   m_to.equals(other.m_to);
        }
        return false;
    }

    public Type getFrom()
    {
        return m_from;
    }

    public Type getTo()
    {
        return m_to;
    }

    /**
     * Implements type equivalence '(R->P)\x = (R->P)'.
    **/
    public Type hide(String x)
    {
        return this;
    }

    public boolean occurs(TypeVariable var)
    {
        return m_from.occurs(var) || m_to.occurs(var);
    }

    public Type reverse()
    {
        return doService(m_from.reverse(), m_to.reverse());
    }

    public void simplifyConstraint(ArrayList list, Type provided)
    {
        TypeService t = provided.lookupService();
        if (null == t) {
            super.simplifyConstraint(list, provided);
        }
        else {
            t.getFrom().simplifyConstraint(list, m_from);
            m_to.simplifyConstraint(list, t.getTo());
        }
    }

    public Type subst(TypeVariable var, Type replace)
    {
        return doService(m_from.subst(var, replace),
                         m_to.subst(var, replace));
    }

    public String toString()
    {
        return "(" + m_from.toString() + "->" + m_to.toString() + ")";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    TypeService(Type from, Type to)
    {
        m_from = from;
        m_to = to;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Type extendLeft(Type left)
    {
        Type t = left.removeService();
        return (left == t) ? super.extendLeft(t) : t.extend(this);
    }

    protected TypeService lookupService()
    {
        return this;
    }

    protected Type removeService()
    {
        return Type.EMPTY;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Type m_from;

    private final Type m_to;

    private Type doService(Type from, Type to)
    {
        return (m_from == from && m_to == to) ? this : from.service(to);
    }

}