package ch.unibe.piccola.vm;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * @author   Stefan Kneubuehl
 * @version  3.4c (Dec 02, 2002)
**/
public class AgentPool
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public AgentPool()
    {
        m_abortRequest = false;
        m_active = new ArrayList();
        m_autoAbort = true;
        m_count = 0;
        m_passive = new ArrayList();
        m_manager = null;
        m_threads = new ThreadGroup("Piccola Threads");
    }

    public synchronized void abort()
    {
        m_abortRequest = true;
        try {
            Thread.sleep(500);
        }
        catch (InterruptedException ex) { }
    }

    public synchronized int activeCount()
    {
        return m_active.size();
    }

    public void setAutoAbort(boolean value)
    {
        m_autoAbort = value;
    }

    public void setObserverFactory(IObserverFactory value)
    {
        m_observerFactory = value;
    }

    public void printThreads()
    {
        Thread[] ts = new Thread[m_threads.activeCount()];
        m_threads.enumerate(ts);
        for (int i = 0; i < ts.length; ++i) {
            System.out.println(ts[i].getName() + " (" + ts[i].getThreadGroup().getName() + ")");
        }
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    IAgentObserver newAgentObserver()
    {
        return null == m_observerFactory ?
            null : m_observerFactory.newAgentObserver();
    }

    IChannelObserver newChannelObserver()
    {
        return null == m_observerFactory ?
            null : m_observerFactory.newChannelObserver();
    }

    synchronized void requestThread(Agent agent)
    {
        AgentThread thread;
        if (m_passive.isEmpty()) {
            thread = new AgentThread(this, m_threads, m_count++);
        }
        else {
            thread = (AgentThread) m_passive.get(0);
            m_passive.remove(0);
        }
        thread.setAgent(agent);
        m_active.add(thread);
        if (null == m_manager) {
            m_manager = new ManagerThread();
            m_manager.start();
        }
    }

    synchronized void releaseThread(AgentThread thread)
    {
        thread.setAgent(null);
        m_active.remove(m_active.indexOf(thread));
        m_passive.add(thread);
    }

    synchronized void removeThread(AgentThread thread)
    {
        m_passive.remove(m_passive.indexOf(thread));
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private boolean m_abortRequest;

    private final ArrayList m_active;

    private boolean m_autoAbort;

    private int m_count;

    private ManagerThread m_manager;

    private IObserverFactory m_observerFactory;

    private final ArrayList m_passive;

    private final ThreadGroup m_threads;

    private synchronized void doAbort()
    {
        Iterator i = m_active.iterator();
        while (i.hasNext()) {
            ((AgentThread) i.next()).abort();
        }
        i = m_passive.iterator();
        while (i.hasNext()) {
            ((AgentThread) i.next()).abort();
        }
        m_manager = null;
    }

    private synchronized boolean noActive()
    {
        return 0 == m_active.size();
    }

    private synchronized int threadCount()
    {
        return m_threads.activeCount();
    }

    private class ManagerThread extends Thread
    {

        public void run()
        {
            while (!m_aborted) {
                try {
                    Thread.sleep(100);
                }
                catch (InterruptedException ex) { }
                if (m_abortRequest || (noActive() && m_autoAbort)) {
                    m_aborted = true;
                    doAbort();
                }
            }
            System.exit(0);
        }

        ManagerThread()
        {
            m_aborted = false;
        }

        private boolean m_aborted;

    }

}