package ch.unibe.piccola.vm;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import ch.unibe.piccola.code.Code;
import ch.unibe.piccola.code.CodeFactory;

/**
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
public class AgentTest extends TestCase
{

    // ------------------------------------------------------------------------
    // test suite
    // ------------------------------------------------------------------------

    public AgentTest(String name)
    {
        super(name);
    }

    public static TestSuite suite()
    {
        return new TestSuite(AgentTest.class);
    }

    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(suite());
    }

    // ------------------------------------------------------------------------
    // test environment
    // ------------------------------------------------------------------------

    public void setUp()
    {
        pool = new VMPool();
        ff = new FormFactory();
        empty = ff.emptyForm();
        pool.setAutoAbort(true);
        cf = new CodeFactory();
        cf.emitExit(null);
        agent = new VM(pool, cf.code(), empty, empty);
        assertSize(1, 1, 1);
    }

    private CodeFactory cf;

    private FormFactory ff;

    private Form empty;

    private VMPool pool;

    private Agent agent;

    private void assertSize(int cs, int ss, int vs)
    {
        assertEquals(cs, agent.sizeCode());
        assertEquals(ss, agent.sizeContext());
        assertEquals(vs, agent.sizeValue());
    }

    // ------------------------------------------------------------------------
    // tests
    // ------------------------------------------------------------------------

    public void testApply()
    {
        cf.emitExit(null);
        agent.opBind("test");
        agent.opPushContext();
        agent.opService(cf.code());
        agent.opPopContext();
        agent.opEmpty();
        assertSize(1, 1, 2);
        agent.opApply();
        assertSize(2, 2, 0);
        assertTrue(agent.getContext().exists("test"));
    }

    public void testBind()
    {
        agent.opBind("label");
        assertSize(1, 1, 1);
        assertTrue(agent.getValue().exists("label"));
    }

    public void testEmpty()
    {
        agent.opEmpty();
        assertSize(1, 1, 2);
        assertTrue(agent.getValue().isEmpty());
    }

    public void testExtend()
    {
        agent.opBind("hello");
        agent.opEmpty();
        agent.opBind("world");
        assertSize(1, 1, 2);
        agent.opExtend();
        assertSize(1, 1, 1);
        assertTrue(agent.getValue().exists("hello"));
        assertTrue(agent.getValue().exists("world"));
    }

}

