package ch.unibe.piccola.vm;

import java.util.ArrayList;
import java.util.SortedMap;
import ch.unibe.piccola.code.CodeFactory;
import ch.unibe.piccola.syntax.Syntax;

/**
 * @author   Stefan Kneubuehl
 * @version  3.6c (Apr 23, 2003)
**/
public class ChannelForm extends DefaultForm
{

    // ------------------------------------------------------------------------
    // private static
    // ------------------------------------------------------------------------

    private static final String LABEL_ARG = Syntax.labelArg();

    private static final String LABEL_DYNAMIC = Syntax.labelDynamic();

    private static final String LABEL_RECEIVE = Syntax.labelReceive();

    private static final String LABEL_SEND = Syntax.labelSend();

    private static int NEXT_ID = 0;

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public boolean equals(Object object)
    {
        return this == object;
    }

    public boolean exists(String label)
    {
        return LABEL_RECEIVE.equals(label) ||
               LABEL_SEND.equals(label);
    }

    public int getId()
    {
        return m_id;
    }

    public Form hide(String label)
    {
        if (exists(label)) {
            return factory().hideForm(label, this);
        }
        else {
            return this;
        }
    }

    public boolean runtimeEquals(Form form)
    {
        return this == form;
    }

    public int runtimeHashCode()
    {
        return 1;
    }

    public String runtimeToString()
    {
        return "(Channel)";
    }

    public String toString()
    {
        return "(Channel)";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    ChannelForm(FormFactory factory, VMPool pool)
    {
        super(factory);
        m_agents = new ArrayList();
        m_buffer = new ArrayList();
        m_id = NEXT_ID++;
        m_observer = pool.newChannelObserver();
        CodeFactory cf = new CodeFactory();
        cf.emitReceive(null, this);
        cf.emitExit(null);
        m_receive = factory().serviceForm(new PiccolaService(pool, cf.code(), factory().emptyForm()));
        cf.clear();
        cf.emitLookup(null, LABEL_DYNAMIC);
        cf.emitProject(null, LABEL_ARG);
        cf.emitSend(null, this);
        cf.emitExit(null);
        m_send = factory().serviceForm(new PiccolaService(pool, cf.code(), factory().emptyForm()));
    }

    final synchronized void send(Form form)
    {
        if (m_agents.isEmpty()) {
            m_buffer.add(form);
        }
        else {
            Agent agent = (Agent) m_agents.get(0);
            agent.pushValue(form);
            agent.unblock();
            m_agents.remove(0);
        }
    }

    final synchronized void listen(Agent agent)
    {
        if (m_buffer.isEmpty()) {
            m_agents.add(agent);
            agent.block();
        }
        else {
            agent.pushValue((Form) m_buffer.get(0));
            m_buffer.remove(0);
        }
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected SortedMap buildBindings()
    {
        SortedMap bindings = super.buildBindings();
        bindings.put(LABEL_RECEIVE, m_receive);
        bindings.put(LABEL_SEND, m_send);
        return bindings;
    }

    public Form projectDef(String label, Form defaultForm)
    {
        if (LABEL_RECEIVE.equals(label)) {
            return m_receive;
        }
        else if (LABEL_SEND.equals(label)) {
            return m_send;
        }
        return defaultForm;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final ArrayList m_agents;

    private final ArrayList m_buffer;

    private final int m_id;

    private final IChannelObserver m_observer;

    private final Form m_receive;

    private final Form m_send;

}

