package ch.unibe.piccola.vm;

import java.util.HashMap;

/**
 * @author   Stefan Kneubuehl
 * @version  4.0 (Jun 01, 2003)
**/
public class FormFactory
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public FormFactory()
    {
        m_empty = new EmptyForm(this);
        m_classForms = new HashMap();
    }

    public Form bindingForm(String label, Form form)
    {
        return new BindingForm(this, label, form);
    }

    public Form channelForm(VMPool pool)
    {
        return new ChannelForm(this, pool);
    }

    public synchronized Form classForm(VMPool pool, Class cl)
    {
        if (!m_classForms.containsKey(cl)) {
            m_classForms.put(cl, new PeerClassForm(this, pool, cl));
        }
        return (Form) m_classForms.get(cl);
    }

    public Form emptyForm()
    {
        return m_empty;
    }

    public Form extensionForm(Form left, Form right)
    {
        return new ExtensionForm(this, left, right);
    }

    public Form externalForm(PeerForm form)
    {
        return new ExternalForm(this, form);
    }

    public Form hideForm(String label, Form form)
    {
        return new HideForm(this, label, form);
    }

    public Form peerForm(VMPool pool, Object object)
    {
        return externalForm((PeerForm) objectForm(pool, object));
    }

    public Form serviceForm(Service service)
    {
        return new ServiceForm(this, service);
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Form m_empty;

    private final HashMap m_classForms;

    private Form objectForm(VMPool pool, Object object)
    {
        return new PeerObjectForm(this, pool, object);
    }

}
