package ch.unibe.piccola.vm;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import ch.unibe.piccola.code.Arguments;

/**
 * Piccola services that map to non-constructor methods.
 *
 * @author   Stefan Kneubuehl
 * @version  3.6c (May 11, 2003)
**/
public class JavaMethod extends JavaOperation
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public Object call(Object target, Arguments args)
    {
        try {
            return get(findMatch(args)).invoke(target, args.toArray());
        }
        catch (IllegalAccessException ex) {
            throw new AgentException(ex);
        }
        catch (IllegalArgumentException ex) {
            throw new AgentException(ex);
        }
        catch (InvocationTargetException ex) {
            throw new AgentException(ex.getTargetException());
        }
    }

    public boolean equals(Object other)
    {
        return this == other;
    }

    public int hashCode()
    {
        return get(0).hashCode();
    }

    public String toString()
    {
        return get(0).getDeclaringClass().getName() + "." + get(0).getName();
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    JavaMethod(Method method)
    {
        m_methods = new ArrayList();
        add(method);
    }

    void add(Method method)
    {
        m_methods.add(method);
        method.setAccessible(true);
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected int count()
    {
        return m_methods.size();
    }

    protected Class[] paramTypes(int index)
    {
        return ((Method) m_methods.get(index)).getParameterTypes();
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private ArrayList m_methods;

    private Method get(int index)
    {
        return (Method) m_methods.get(index);
    }

}

