package ch.unibe.piccola.vm;

import java.lang.reflect.Field;
import java.util.Iterator;
import java.util.SortedMap;
import ch.unibe.piccola.code.CodeFactory;
import ch.unibe.piccola.code.JavaOp;

/**
 * @author   Stefan Kneubuehl
 * @version  3.6c (May 11, 2003)
**/
public abstract class PeerForm extends DefaultForm
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public boolean equals(Object object)
    {
        if (object instanceof Form) {
            return runtimeEquals((Form) object);
        }
        return false;
    }

    public boolean exists(String label)
    {
        return ats().containsKey(label) ||
               ops().containsKey(label);
    }

    public Form hide(String label)
    {
        return factory().hideForm(label, this);
    }

    public boolean runtimeEquals(Form form)
    {
        return m_object.equals(form.down());
    }

    public int runtimeHashCode()
    {
        return m_object.hashCode();
    }

    public String runtimeToString()
    {
        return m_object.toString();
    }

    public String toString()
    {
        return runtimeToString();
    }

    public Object unwrap()
    {
        return m_object;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected PeerForm(FormFactory factory, VMPool pool, Object object)
    {
        super(factory);
        m_object = object;
        m_pool = pool;
    }

    protected abstract SortedMap ats();

    protected abstract SortedMap ops();

    protected SortedMap buildBindings()
    {
        SortedMap bindings = super.buildBindings();
        String label;
        Iterator i = ats().keySet().iterator();
        while (i.hasNext()) {
            label = (String) i.next();
            bindings.put(label, lookupField(label));
        }
        i = ops().keySet().iterator();
        while (i.hasNext()) {
            label = (String) i.next();
            bindings.put(label, lookupService(label));
        }
        return bindings;
    }

    public Form projectDef(String label, Form defaultForm)
    {
        if (ats().containsKey(label)) {
            return lookupField(label);
        }
        if (ops().containsKey(label)) {
            return lookupService(label);
        }
        return defaultForm;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private Object m_object;

    private VMPool m_pool;

    private Form lookupService(String label)
    {
        CodeFactory cf = new CodeFactory();
        cf.clear();
        cf.emitJavaCall(null, (JavaOp) ops().get(label), m_object);
        cf.emitExit(null);
        return m_pool.m_factory.serviceForm(new PiccolaService(m_pool, cf.code(), factory().emptyForm()));
    }

    private Form lookupField(String label)
    {
        try {
            return m_pool.up(((Field) ats().get(label)).get(m_object));
        }
        catch (IllegalAccessException ex) {
            throw new AgentException(ex);
        }
    }

}
