package ch.unibe.piccola.vm;

import java.util.HashMap;
import java.io.OutputStream;
import java.io.PrintStream;
import ch.unibe.piccola.code.Code;
import ch.unibe.piccola.code.CodeFactory;
import ch.unibe.piccola.syntax.Syntax;
import ch.unibe.piccola.vm.Form;

/**
 * @author   Stefan Kneubuehl
 * @version  3.6c (May 11, 2003)
**/
public class VMPool extends AgentPool
{

    // ------------------------------------------------------------------------
    // private static
    // ------------------------------------------------------------------------

    private static final String LABEL_ARG = Syntax.labelArg();

    private static final String LABEL_DYNAMIC = Syntax.labelDynamic();

    private static final String LABEL_PEER = Syntax.labelPeer();

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public VMPool()
    {
        m_piccolaWrapperMap = new HashMap();
        m_factory = new FormFactory();
        m_callDynamic = m_factory.emptyForm();
        m_out = System.out;
    }

    public Form newChannel()
    {
        return m_factory.channelForm(this);
    }

    public Form newClass(Class cl)
    {
        return m_factory.externalForm((PeerClassForm) peerClass(cl));
    }

    public Form newClass(String className) throws ClassNotFoundException
    {
        Class cl;
        if (className.equals("boolean")) { cl = Boolean.TYPE; }
        else if (className.equals("byte")) { cl = Byte.TYPE; }
        else if (className.equals("char")) { cl = Character.TYPE; }
        else if (className.equals("double")) { cl = Double.TYPE; }
        else if (className.equals("float")) { cl = Float.TYPE; }
        else if (className.equals("int")) { cl = Integer.TYPE; }
        else if (className.equals("long")) { cl = Long.TYPE; }
        else if (className.equals("short")) { cl =  Short.TYPE; }
        else cl = Class.forName(className);
        return newClass(cl);
    }

    public Form newMetaService()
    {
        CodeFactory cf = new CodeFactory();
        cf.emitLookup(null, LABEL_DYNAMIC);
        cf.emitProject(null, LABEL_ARG);
        cf.emitMeta(null);
        cf.emitExit(null);
        return m_factory.serviceForm(new PiccolaService(this, cf.code(), m_factory.emptyForm()));
    }

    public Form wrap(Object object)
    {
        return m_factory.peerForm(this, object);
    }

    public Form newService(Code code, Form context)
    {
        return m_factory.serviceForm(new PiccolaService(this, code, context));
    }

    public Form newVariable(Form initialValue)
    {
        Form var = up(new Variable(initialValue));
        return var.extend(var.project("get"));
    }

    public void registerPiccolaWrapper(String className, Form wrapper)
    {
        if (className.equals("array")) {
            m_arrayWrapper = wrapper;
        }
        else {
            try {
                m_piccolaWrapperMap.put(Class.forName(className), wrapper);
            }
            catch (ClassNotFoundException ex) { }
        }
    }

    public void setExceptionHandler(Form value)
    {
        m_exceptionHandler = value;
    }

    public void setCallDynamic(Form value)
    {
        m_callDynamic = value;
    }

    public void print(String str)
    {
        if (null != m_out) { m_out.print(str); }
    }

    public void println(String str)
    {
        if (null != m_out) { m_out.println(str); }
    }

    /**
     * Returns the current Piccola standard output.
     * The default output stream is <code>System.out</code>.
     *
     * @return the current Piccola standard output.
    **/
    public final PrintStream getOut()
    {
        return m_out;
    }

    /**
     * Sets the current Piccola standard output. Set output to <code>null</code>
     * to ignore the output. The default output stream is <code>System.out</code>.
     *
     * @param out the OutputStream to become the Piccola standard output.
    **/
    public final void setOut(OutputStream out)
    {
        m_out = new PrintStream(out);
    }

    /**
     * Sets the current Piccola standard output. Set output to <code>null</code>
     * to ignore the output. The default output stream is <code>System.out</code>.
     *
     * @param out the PrintStream to become the Piccola standard output.
    **/
    public void setOut(PrintStream value)
    {
        m_out = value;
    }

    public Form up(Object object)
    {
        if (null == object) {
            return m_factory.emptyForm();
        }
        else if (object instanceof Form) {
            return (Form) object;
        }
        else {
            return wrap(object);
        }
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    Form peerClass(Class cl)
    {
        return m_factory.classForm(this, cl);
    }

    Form getExceptionHandler()
    {
        return m_exceptionHandler;
    }

    Form getPiccolaWrapper(Class cl)
    {
        if (cl.isArray()) {
            return m_arrayWrapper;
        }
        Form wrapper;
        do {
            wrapper = (Form) m_piccolaWrapperMap.get(cl);
            cl = cl.getSuperclass();
        } while (null == wrapper && null != cl);
        return wrapper;
    }

    Form getCallDynamic()
    {
        return m_callDynamic;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private Form m_exceptionHandler;

    protected FormFactory m_factory;

    private final HashMap m_piccolaWrapperMap;

    private Form m_arrayWrapper;

    private Form m_callDynamic;

    private PrintStream m_out;

}

